import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * The super class.
 * 
 * @author Ninto
 * @version 0.2
 */
abstract public class superClass  extends Actor
{
    /**X- and Y-coordinates, and horizontally- and vertical-speed.*/
    private double x, y, a, b;
    /**
     * The force of the gravity and the ground-friction.
     * @see addedToWorld(World)
     */
    private double gravity, groundFriction;
    /**Whether this is solid or not*/
    private boolean solid;
    
    /**
     * Create a solid superClass.
     * @see superClass(int, double)<br>
     * @see superClass(double, double)
     */
    public superClass() {
        solid = true;
    }
    
    /**
     * Create a superClass.
     * @param dir The direction of the new superClass.
     * @param speed The speed of the new superClass.
     * @see superClass(double, double)
     */
    public superClass(int dir, double speed) {
        double angle = Math.toRadians(dir);
        a = Math.cos(angle) * speed;
        b = Math.sin(angle) * speed;
        solid = false;
    }
    
    /**
     * Create a superClass.
     * @param hspped The horizontally-speed.
     * @param vspped The vertical-speed.
     * @see superClass(int, double)
     */
    public superClass(double hspeed, double vspeed) {
        a = hspeed;
        b = vspeed;
        solid = false;
    }
    
    /**
     * Get the world in a 'myWorld'format instead of the normal World-format.
     * @return The myWorld that this superClass lives in.
     */
    public final myWorld getWorld() {
        return (myWorld)super.getWorld();
    }
    
    /**
     * Set the x, y, gravity and groundFriction values.
     */
    public final void addedToWorld(World world) {
        x = getX();
        y = getY();
        gravity = myWorld.gravity;
        groundFriction = myWorld.groundFriction;
    }
    
    /**
     * Move by use the diferent speeds and forces.
     */
    public final void move() {
        x += a;
        y += b;
        setLocation(x, y);
        collide();
        b += gravity;
        if(onGround()) {
            if(getHSpeed() < 0) setSpeed(getHSpeed()+groundFriction, getVSpeed());
            if(getHSpeed() > 0) setSpeed(getHSpeed()-groundFriction, getVSpeed());
        }
    }
    
    /**
     * Set location with doubles.
     */
    public final void setLocation(double x, double y) {
        super.setLocation((int)Math.round(x), (int)Math.round(y));
        this.x = getX();
        this.y = getY();
    }
    
    /**
     * @see setlocation(double, double)
     */
    public final void setLocation(int x, int y) {
        setLocation((double)x, (double)y);
    }
    
    /**
     * Return the horizontally speed of this.
     */
    public final double getHSpeed() {
        if(solid) return 0;
        return a;
    }
    
    /**
     * Return the vertical speed of this.
     */
    public final double getVSpeed() {
        if(solid) return 0;
        return b;
    }
    
    /**
     * Set the speed.
     * @param hspeed The horizontally speed.
     * @param vspeed The vertical speed.
     */
    public final void setSpeed(double hspeed, double vspeed) {
        a = hspeed;
        b = vspeed;
    }
    
    /**
     * Return the direction of this to move.
     */
    public final int getDirection() {
        return (int)Math.round(Math.toDegrees((Math.atan2(b, a))));
    }
    
    /**
     * Replace this with another superClass-object.
     * @param object The other superClass-object.
     */
    public final void replaceWith(superClass object) {
        getWorld().addObject(object, getX(), getY());
        getWorld().removeObject(this);
    }
    
    /**
     * Check however this is out of the limits of the world or not.
     * <br>(This will also remove it too so whenever you call this should you say: 'if(isOutOfWorld()) return;').
     */
    public final boolean isOutOfWorld() {
        if(getX() == 0 || getX() == getWorld().getWidth()-1) {
            getWorld().removeObject(this);
            return true;
        }
        return false;
    }
    
    /**
     * A method that i found in the ant scenario, that generates a random chance of something.
     */
    public static final boolean randomChance(double percent)
    {
        return Greenfoot.getRandomNumber(1000) < percent*10;
    }
    
    /**
     * Check whether this is on ground or not.
     * <br><i>This should be overriten later.</i>
     * @return Whether this is on ground or not.
     */
    public boolean onGround() {
        if(getY() >= getWorld().getHeight()-1) return true;
        return false;
    }
    
    /**
     * Collide with a intersecting object (if any).
     */
    private void collide() {
        superClass target = (superClass)getOneIntersectingObject(superClass.class);
        if(target != null) {
            double hspeed = getHSpeed();
            double vspeed = getVSpeed();
            setSpeed(target.getHSpeed(), target.getVSpeed());
            target.setSpeed(hspeed, vspeed);
        }
    }
}